@@ -9,9 +9,13 @@ module RSpec
         config.add_setting :verbose_retry, :default => false
         config.add_setting :default_retry_count, :default => 1
         config.add_setting :default_sleep_interval, :default => 0
+        config.add_setting :exponential_backoff, :default => false
         config.add_setting :clear_lets_on_failure, :default => true
         config.add_setting :display_try_failure_messages, :default => false
 
+        # retry based on example metadata
+        config.add_setting :retry_count_condition, :default => ->(_) { nil }
+
         # If a list of exceptions is provided and 'retry' > 1, we only retry if
         # the exception that was raised by the example is NOT in that list. Otherwise
         # we ignore the 'retry' value and fail immediately.
@@ -43,11 +47,8 @@ module RSpec
       current_example.attempts ||= 0
     end
 
-    # context.example is deprecated, but RSpec.current_example is not
-    # available until RSpec 3.0.
     def current_example
-      RSpec.respond_to?(:current_example) ?
-        RSpec.current_example : @ex.example
+      @current_example ||= RSpec.current_example
     end
 
     def retry_count
@@ -55,6 +56,7 @@ module RSpec
           (
           ENV['RSPEC_RETRY_RETRY_COUNT'] ||
               ex.metadata[:retry] ||
+              RSpec.configuration.retry_count_condition.call(ex) ||
               RSpec.configuration.default_retry_count
           ).to_i,
           1
@@ -76,8 +78,12 @@ module RSpec
     end
 
     def sleep_interval
-      ex.metadata[:retry_wait] ||
-          RSpec.configuration.default_sleep_interval
+      if ex.metadata[:exponential_backoff]
+          2**(current_example.attempts-1) * ex.metadata[:retry_wait]
+      else
+          ex.metadata[:retry_wait] ||
+              RSpec.configuration.default_sleep_interval
+      end
     end
 
     def exceptions_to_hard_fail
@@ -111,6 +117,9 @@ module RSpec
           end
         end
 
+        example.metadata[:retry_attempts] = self.attempts
+        example.metadata[:retry_exceptions] ||= []
+
         example.clear_exception
         ex.run
 
@@ -118,6 +127,8 @@ module RSpec
 
         break if example.exception.nil?
 
+        example.metadata[:retry_exceptions] << example.exception
+
         break if attempts >= retry_count
 
         if exceptions_to_hard_fail.any?
@@ -149,7 +160,7 @@ module RSpec
           example.example_group_instance.instance_exec(example, &RSpec.configuration.retry_callback)
         end
 
-        sleep sleep_interval if sleep_interval.to_i > 0
+        sleep sleep_interval if sleep_interval.to_f > 0
       end
     end
 
