@@ -56,6 +56,15 @@ describe RSpec::Retry do
       end
     end
 
+    context 'with lambda condition' do
+      before(:all) { set_expectations([false, true]) }
+
+      it "should get retry count from condition call", retry_me_once: true do
+        expect(true).to be(shift_expectation)
+        expect(count).to eq(2)
+      end
+    end
+
     context 'with :retry => 0' do
       after(:all) { @@this_ran_once = nil }
       it 'should still run once', retry: 0 do
@@ -83,6 +92,21 @@ describe RSpec::Retry do
       end
     end
 
+    context "with exponential backoff enabled", :retry => 3, :retry_wait => 0.001, :exponential_backoff => true do
+      context do
+        before(:all) do
+          set_expectations([false, false, true])
+          @start_time = Time.now
+        end
+
+        it 'should run example until :retry times', :retry => 3 do
+          expect(true).to be(shift_expectation)
+          expect(count).to eq(3)
+          expect(Time.now - @start_time).to be >= (0.001)
+        end
+      end
+    end
+
     describe "with a list of exceptions to immediately fail on", :retry => 2, :exceptions_to_hard_fail => [HardFailError] do
       context "the example throws an exception contained in the hard fail list" do
         it "does not retry" do
@@ -109,6 +133,51 @@ describe RSpec::Retry do
     end
 
     describe "with a list of exceptions to retry on", :retry => 2, :exceptions_to_retry => [RetryError] do
+      context do
+        let(:rspec_version) { RSpec::Core::Version::STRING }
+
+        let(:example_code) do
+          %{
+            $count ||= 0
+            $count += 1
+
+            raise NameError unless $count > 2
+          }
+        end
+
+        let!(:example_group) do
+          $count, $example_code = 0, example_code
+
+          RSpec.describe("example group", exceptions_to_retry: [NameError], retry: 3).tap do |this|
+            this.run # initialize for rspec 3.3+ with no examples
+          end
+        end
+
+        let(:retry_attempts) do
+          example_group.examples.first.metadata[:retry_attempts]
+        end
+
+        it 'should retry and match attempts metadata' do
+          example_group.example { instance_eval($example_code) }
+          example_group.run
+
+          expect(retry_attempts).to eq(2)
+        end
+
+        let(:retry_exceptions) do
+          example_group.examples.first.metadata[:retry_exceptions]
+        end
+
+        it 'should add exceptions into retry_exceptions metadata array' do
+          example_group.example { instance_eval($example_code) }
+          example_group.run
+
+          expect(retry_exceptions.count).to eq(2)
+          expect(retry_exceptions[0].class).to eq NameError
+          expect(retry_exceptions[1].class).to eq NameError
+        end
+      end
+
       context "the example throws an exception contained in the retry list" do
         it "retries the maximum number of times" do
           raise RetryError unless count > 1
@@ -263,6 +332,37 @@ describe RSpec::Retry do
     end
   end
 
+  describe 'Example::Procsy#attempts' do
+    let!(:example_group) do
+      RSpec.describe do
+        before :all do
+          @@results = {}
+        end
+
+        around do |example|
+          example.run_with_retry
+          @@results[example.description] = [example.exception.nil?, example.attempts]
+        end
+
+        specify 'without retry option' do
+          expect(true).to be(true)
+        end
+
+        specify 'with retry option', retry: 3 do
+          expect(true).to be(false)
+        end
+      end
+    end
+
+    it 'should be exposed' do
+      example_group.run
+      expect(example_group.class_variable_get(:@@results)).to eq({
+        'without retry option' => [true, 1],
+        'with retry option' => [false, 3]
+      })
+    end
+  end
+
   describe 'output in verbose mode' do
 
     line_1 = __LINE__ + 8
